@@ -31,6 +31,8 @@
 #include "banshee-player-video.h"
 #include "banshee-player-equalizer.h"
 #include "banshee-player-missing-elements.h"
+#include "banshee-player-replaygain.h"
+#include "banshee-player-vis.h"
 
 // ---------------------------------------------------------------------------
 // Private Functions
@@ -51,8 +53,12 @@ bp_pipeline_process_tag (const GstTagList *tag_list, const gchar *tag_name, Bans
     
     value = gst_tag_list_get_value_index (tag_list, tag_name, 0);
 
-    if (value != NULL && player->tag_found_cb != NULL) {
-        player->tag_found_cb (player, tag_name, value);
+    if (value != NULL) {
+        _bp_replaygain_process_tag (player, tag_name, value);
+    
+        if (player->tag_found_cb != NULL) {
+            player->tag_found_cb (player, tag_name, value);
+        }
     }
 }
 
@@ -77,6 +83,7 @@ bp_pipeline_bus_callback (GstBus *bus, GstMessage *message, gpointer userdata)
             gst_message_parse_state_changed (message, &old, &new, &pending);
             
             _bp_missing_elements_handle_state_changed (player, old, new);
+            _bp_replaygain_handle_state_changed (player, old, new, pending);
             
             if (player->state_changed_cb != NULL) {
                 player->state_changed_cb (player, old, new, pending);
@@ -189,9 +196,14 @@ _bp_pipeline_construct (BansheePlayer *player)
     // fall back on auto, which should work on windows, and as a last ditch, try alsa
     audiosink = gst_element_factory_make ("gconfaudiosink", "audiosink");
     if (audiosink == NULL) {
-        audiosink = gst_element_factory_make ("autoaudiosink", "audiosink");
-        if (audiosink == NULL) {
-            audiosink = gst_element_factory_make ("alsasink", "audiosink");
+        audiosink = gst_element_factory_make ("directsoundsink", "audiosink");
+        if (audiosink != NULL) {
+            g_object_set (G_OBJECT (audiosink), "volume", 1.0, NULL);
+        } else {
+            audiosink = gst_element_factory_make ("autoaudiosink", "audiosink");
+            if (audiosink == NULL) {
+                audiosink = gst_element_factory_make ("alsasink", "audiosink");
+            }
         }
     }
     
@@ -239,10 +251,6 @@ _bp_pipeline_construct (BansheePlayer *player)
     gst_element_add_pad (player->audiobin, gst_ghost_pad_new ("sink", teepad));
     gst_object_unref (teepad);
 
-    // Link the first tee pad to the primary audio sink queue
-    gst_pad_link (gst_element_get_request_pad (player->audiotee, "src0"), 
-        gst_element_get_pad (audiosinkqueue, "sink"));
-
     // Link the queue and the actual audio sink
     if (player->equalizer != NULL) {
         // link in equalizer, preamp and audioconvert.
@@ -253,6 +261,8 @@ _bp_pipeline_construct (BansheePlayer *player)
         gst_element_link (audiosinkqueue, audiosink);
     }
     
+    _bp_vis_pipeline_setup (player);
+    
     // Now that our internal audio sink is constructed, tell playbin to use it
     g_object_set (G_OBJECT (player->playbin), "audio-sink", player->audiobin, NULL);
     
@@ -264,6 +274,22 @@ _bp_pipeline_construct (BansheePlayer *player)
     _bp_cdda_pipeline_setup (player);
     _bp_video_pipeline_setup (player, bus);
 
+    // This call must be the last one in the pipeline setup to work around a
+    // GStreamer 0.10.21-0.10.22 algorithm that causes the last-allocated pad
+    // to be the one used for buffer allocations.  If the visualization one
+    // winds up being used for that then the pipeline will freeze when
+    // visualizations are disabled.
+    //
+    // When 0.10.23 is more mainstream we can use the new alloc-pad property to
+    // force selection of this pad for allocation.  Until then we just have to
+    // make sure it's the last one allocated.
+    //
+    // -- Chris Howie <cdhowie@gmail.com>
+
+    // Link the first tee pad to the primary audio sink queue
+    gst_pad_link (gst_element_get_request_pad (player->audiotee, "src0"),
+        gst_element_get_pad (audiosinkqueue, "sink"));
+
     return TRUE;
 }
 
@@ -282,5 +308,7 @@ _bp_pipeline_destroy (BansheePlayer *player)
         gst_object_unref (GST_OBJECT (player->playbin));
     }
     
+    _bp_vis_pipeline_destroy (player);
+    
     player->playbin = NULL;
 }
