@@ -39,8 +39,6 @@ namespace Banshee.GnomeBackend
 {
     public class GConfConfigurationClient : IConfigurationClient
     {
-        private static string base_key = "/apps/banshee-1/";
-        
         private GConf.Client client;
         private Dictionary<string, string> key_table = new Dictionary<string, string> ();
         
@@ -58,21 +56,35 @@ namespace Banshee.GnomeBackend
             }
         }
 
+        private static string base_key;
+        private static string BaseKey {
+            get {
+                if (base_key == null) {
+                    base_key = ApplicationContext.CommandLine["gconf-base-key"];
+                    if (!base_key.StartsWith ("/apps/") || !base_key.EndsWith ("/")) {
+                        Log.Debug ("Using default gconf-base-key");
+                        base_key = "/apps/banshee-1/";
+                    }
+                }
+                return base_key;
+            }
+        }
+
         private string CreateKey (string @namespace, string part)
         {
-            string hash_key = String.Format ("{0}{1}", @namespace, part);
+            string hash_key = String.Concat (@namespace, part);
             lock (((ICollection)key_table).SyncRoot) {
                 if (!key_table.ContainsKey (hash_key)) {
                     part = part.Replace ('/', '_');
                     if (@namespace == null) {
-                        key_table.Add (hash_key, String.Format ("{0}{1}", base_key, StringUtil.CamelCaseToUnderCase (part)));
+                        key_table.Add (hash_key, String.Concat (BaseKey, StringUtil.CamelCaseToUnderCase (part)));
                     } else if (@namespace.StartsWith ("/")) {
-                        key_table.Add (hash_key, String.Format ("{0}{1}{2}", @namespace,
+                        key_table.Add (hash_key, String.Concat (@namespace,
                             @namespace.EndsWith ("/") ? String.Empty : "/", StringUtil.CamelCaseToUnderCase (part)));
                     } else {
                         @namespace = @namespace.Replace ('/', '_');
-                        key_table.Add (hash_key, String.Format ("{0}{1}",
-                            base_key, StringUtil.CamelCaseToUnderCase (String.Format ("{0}/{1}", @namespace.Replace (".", "/"), part))
+                        key_table.Add (hash_key, String.Concat (BaseKey,
+                            StringUtil.CamelCaseToUnderCase (String.Concat (@namespace.Replace (".", "/"), "/", part))
                         ));
                     }
 
@@ -110,7 +122,10 @@ namespace Banshee.GnomeBackend
             
             try {
                 return (T)client.Get (CreateKey (@namespace, key));
-            } catch {
+            } catch (GConf.NoSuchKeyException) {
+                return fallback;
+            } catch (Exception e) {
+                Log.Exception (String.Format ("Could no read GConf key {0}.{1}", @namespace, key), e);
                 return fallback;
             }
         }
