@@ -35,11 +35,19 @@ namespace Banshee.Configuration
         public static SchemaEntry<T> Zero;
     
         public SchemaEntry (string @namespace, string key, T defaultValue, 
+            string shortDescription, string longDescription) :
+            this (@namespace, key, defaultValue, default(T), default(T), shortDescription, longDescription)
+        {
+        }
+
+        public SchemaEntry (string @namespace, string key, T defaultValue, T minValue, T maxValue,
             string shortDescription, string longDescription)
         {
             Namespace = @namespace;
             Key = key;
             DefaultValue = defaultValue;
+            MinValue = minValue;
+            MaxValue = maxValue;
             ShortDescription = shortDescription;
             LongDescription = longDescription;
         }
@@ -54,14 +62,24 @@ namespace Banshee.Configuration
             return ConfigurationClient.Get<T> (this, fallback);
         }
 
-        public void Set (T value)
+        public bool Set (T value)
         {
+            if (!Object.Equals (MinValue, default (T)) || !Object.Equals(MaxValue, default (T))) {
+                if (((IComparable<T>)MinValue).CompareTo (value) > 0 ||
+                    ((IComparable<T>)MaxValue).CompareTo (value) < 0) {
+                    return false;
+                }
+            }
+
             ConfigurationClient.Set<T> (this, value);
+            return true;
         }
 
         public readonly string Namespace;
         public readonly string Key;
         public readonly T DefaultValue;
+        public readonly T MinValue;
+        public readonly T MaxValue;
         public readonly string ShortDescription;
         public readonly string LongDescription;
         
