@@ -53,7 +53,8 @@ namespace Banshee.Streaming
                     throw new TagLib.UnsupportedFormatException ("File does not contain video or audio");
                 }
                 return file;
-            } catch (Exception) {
+            } catch (Exception e) {
+                Hyena.Log.DebugException (e);
                 return null;
             }
         }
@@ -88,8 +89,6 @@ namespace Banshee.Streaming
         
         private static void FindTrackMediaAttributes (TrackInfo track, TagLib.File file)
         {
-            track.MediaAttributes = track.MediaAttributes;
-            
             if ((file.Properties.MediaTypes & TagLib.MediaTypes.Audio) != 0) {
                 track.MediaAttributes |= TrackMediaAttributes.AudioStream;
             }
@@ -98,14 +97,30 @@ namespace Banshee.Streaming
                 track.MediaAttributes |= TrackMediaAttributes.VideoStream;
             }
             
+            if (file.Tag.FirstGenre == "Podcast" || file.Tag.Album == "Podcast") {
+                track.MediaAttributes |= TrackMediaAttributes.Podcast;
+            }
+
             // TODO: Actually figure out, if possible at the tag/file level, if
             // the file is actual music, podcast, audiobook, movie, tv show, etc.
-            // For now just assume that if it's only audio, it's music, since that's
-            // what we've just historically assumed on any media type
-            if ((track.MediaAttributes & TrackMediaAttributes.VideoStream) == 0) {
+            // For now just assume that if it's only audio and not podcast, it's
+            // music, since that's what we've just historically assumed on any media type
+            if (!track.HasAttribute (TrackMediaAttributes.VideoStream) &&
+                !track.HasAttribute (TrackMediaAttributes.Podcast)) {
                 track.MediaAttributes |= TrackMediaAttributes.Music;
             }
         }
+
+        public static void TrackInfoMerge (TrackInfo track, SafeUri uri)
+        {
+            track.Uri = uri;
+            TagLib.File file = StreamTagger.ProcessUri (uri);
+            TrackInfoMerge (track, file);
+
+            if (file == null) {
+                track.TrackTitle = uri.AbsoluteUri;
+            }
+        }
         
         public static void TrackInfoMerge (TrackInfo track, TagLib.File file)
         {
@@ -122,17 +137,21 @@ namespace Banshee.Streaming
             if (file != null) {
                 track.Uri = new SafeUri (file.Name);
                 track.MimeType = file.MimeType;
-                track.BitRate  = file.Properties.AudioBitrate;
                 track.Duration = file.Properties.Duration;
+                track.BitRate  = file.Properties.AudioBitrate;
                 
                 FindTrackMediaAttributes (track, file);
     
                 track.ArtistName = Choose (file.Tag.JoinedPerformers, track.ArtistName, preferTrackInfo);
+                track.ArtistNameSort = Choose (file.Tag.JoinedPerformersSort, track.ArtistNameSort, preferTrackInfo);
                 track.AlbumTitle = Choose (file.Tag.Album, track.AlbumTitle, preferTrackInfo);
+                track.AlbumTitleSort = Choose (file.Tag.AlbumSort, track.AlbumTitleSort, preferTrackInfo);
                 track.AlbumArtist = Choose (file.Tag.FirstAlbumArtist, track.AlbumArtist, preferTrackInfo);
-                track.IsCompilation = IsCompilation (file.Tag);
+                track.AlbumArtistSort = Choose (file.Tag.FirstAlbumArtistSort, track.AlbumArtistSort, preferTrackInfo);
+                track.IsCompilation = IsCompilation (file);
                 
                 track.TrackTitle = Choose (file.Tag.Title, track.TrackTitle, preferTrackInfo);
+                track.TrackTitleSort = Choose (file.Tag.TitleSort, track.TrackTitleSort, preferTrackInfo);
                 track.Genre = Choose (file.Tag.FirstGenre, track.Genre, preferTrackInfo);
                 track.Composer = Choose (file.Tag.FirstComposer, track.Composer, preferTrackInfo);
                 track.Conductor = Choose (file.Tag.Conductor, track.Conductor, preferTrackInfo);
@@ -148,7 +167,7 @@ namespace Banshee.Streaming
                 track.Bpm = Choose ((int)file.Tag.BeatsPerMinute, track.Bpm, preferTrackInfo);
             } else {
                 track.MediaAttributes = TrackMediaAttributes.AudioStream;
-                if (VideoExtensions.IsMatchingFile (track.Uri.LocalPath)) {
+                if (track.Uri != null && VideoExtensions.IsMatchingFile (track.Uri.LocalPath)) {
                     track.MediaAttributes = TrackMediaAttributes.VideoStream;
                 }
             }
@@ -171,22 +190,97 @@ namespace Banshee.Streaming
             // TODO these ideas could also be done in an extension that collects such hacks
         }
             
-        private static bool IsCompilation (TagLib.Tag tag)
+        private static bool IsCompilation (TagLib.File file)
         {
-            TagLib.Id3v2.Tag id3v2_tag = tag as TagLib.Id3v2.Tag;
-            if (id3v2_tag != null && id3v2_tag.IsCompilation)
-                return true;
+            try {
+                TagLib.Id3v2.Tag id3v2_tag = file.GetTag(TagLib.TagTypes.Id3v2, true) as TagLib.Id3v2.Tag;
+                if (id3v2_tag != null && id3v2_tag.IsCompilation)
+                    return true;
+            } catch {}
 
-            TagLib.Mpeg4.AppleTag apple_tag = tag as TagLib.Mpeg4.AppleTag;
-            if (apple_tag != null && apple_tag.IsCompilation)
-                return true;
+            try {
+                TagLib.Mpeg4.AppleTag apple_tag = file.GetTag(TagLib.TagTypes.Apple,true) as TagLib.Mpeg4.AppleTag;
+                if (apple_tag != null && apple_tag.IsCompilation)
+                    return true;
+            } catch {}
             
-            if (tag.Performers.Length > 0 && tag.AlbumArtists.Length > 0 &&
-                (tag.Performers.Length != tag.AlbumArtists.Length || tag.FirstAlbumArtist != tag.FirstPerformer)) {
+            // FIXME the FirstAlbumArtist != FirstPerformer check might return true for half the
+            // tracks on a compilation album, but false for some
+            // TODO checked for 'Soundtrack' (and translated) in the title?
+            if (file.Tag.Performers.Length > 0 && file.Tag.AlbumArtists.Length > 0 &&
+                (file.Tag.Performers.Length != file.Tag.AlbumArtists.Length ||
+                file.Tag.FirstAlbumArtist != file.Tag.FirstPerformer)) {
                 return true;
             }
             return false;
         }
+
+        private static void SaveIsCompilation (TagLib.File file, bool is_compilation)
+        {
+            try {
+                TagLib.Id3v2.Tag id3v2_tag = file.GetTag(TagLib.TagTypes.Id3v2, true) as TagLib.Id3v2.Tag;
+                if (id3v2_tag != null) {
+                    id3v2_tag.IsCompilation = is_compilation;
+                    return;
+                }
+            } catch {}
+
+            try {
+                TagLib.Mpeg4.AppleTag apple_tag = file.GetTag(TagLib.TagTypes.Apple,true) as TagLib.Mpeg4.AppleTag;
+                if (apple_tag != null) {
+                    apple_tag.IsCompilation = is_compilation;
+                    return;
+                }
+            } catch {}
+        }
+
+        public static bool SaveToFile (TrackInfo track)
+        {
+            // FIXME taglib# does not seem to handle writing metadata to video files well at all atm
+            // so not allowing
+            if ((track.MediaAttributes & TrackMediaAttributes.VideoStream) != 0) {
+                Hyena.Log.DebugFormat ("Avoiding 100% cpu bug with taglib# by not writing metadata to video file {0}", track);
+                return false;
+            }
+        
+            // Note: this should be kept in sync with the metadata read in StreamTagger.cs
+            TagLib.File file = ProcessUri (track.Uri);
+            if (file == null) {
+                return false;
+            }
+            
+            file.Tag.Performers = new string [] { track.ArtistName };
+            file.Tag.PerformersSort = new string [] { track.ArtistNameSort };
+            file.Tag.Album = track.AlbumTitle;
+            file.Tag.AlbumSort = track.AlbumTitleSort;
+            file.Tag.AlbumArtists = track.AlbumArtist == null ? new string [0] : new string [] {track.AlbumArtist};
+            file.Tag.AlbumArtistsSort = (track.AlbumArtistSort == null ? new string [0] : new string [] {track.AlbumArtistSort});
+            // Bug in taglib-sharp-2.0.3.0: Crash if you send it a genre of "{ null }"
+            // on a song with both ID3v1 and ID3v2 metadata. It's happy with "{}", though.
+            // (see http://forum.taglib-sharp.com/viewtopic.php?f=5&t=239 )
+            file.Tag.Genres = (track.Genre == null) ? new string[] {} : new string [] { track.Genre };
+            file.Tag.Title = track.TrackTitle;
+            file.Tag.TitleSort = track.TrackTitleSort;
+            file.Tag.Track = (uint)track.TrackNumber;
+            file.Tag.TrackCount = (uint)track.TrackCount;
+            file.Tag.Composers = new string [] { track.Composer };
+            file.Tag.Conductor = track.Conductor;
+            file.Tag.Grouping = track.Grouping;
+            file.Tag.Copyright = track.Copyright;
+            file.Tag.Comment = track.Comment;
+            file.Tag.Disc = (uint)track.DiscNumber;
+            file.Tag.DiscCount = (uint)track.DiscCount;
+            file.Tag.Year = (uint)track.Year;
+            file.Tag.BeatsPerMinute = (uint)track.Bpm;
+            
+            SaveIsCompilation (file, track.IsCompilation);
+            file.Save ();
+
+            track.FileSize = Banshee.IO.File.GetSize (track.Uri);
+            track.FileModifiedStamp = Banshee.IO.File.GetModifiedTime (track.Uri);
+            track.LastSyncedStamp = DateTime.Now;
+            return true;
+        }
     
         public static void TrackInfoMerge (TrackInfo track, StreamTag tag)
         {
@@ -195,6 +289,10 @@ namespace Banshee.Streaming
                     case CommonTags.Artist:
                         track.ArtistName = Choose ((string)tag.Value, track.ArtistName);
                         break;
+                    case CommonTags.ArtistSortName:
+                    case CommonTags.MusicBrainzSortName:
+                        track.ArtistNameSort = Choose ((string)tag.Value, track.ArtistNameSort);
+                        break;
                     case CommonTags.Title:
                         //track.TrackTitle = Choose ((string)tag.Value, track.TrackTitle);
                         string title = Choose ((string)tag.Value, track.TrackTitle);
@@ -219,9 +317,15 @@ namespace Banshee.Streaming
                             track.TrackTitle = title;
                         }
                         break;
+                    case CommonTags.TitleSortName:
+                        track.TrackTitleSort = Choose ((string)tag.Value, track.TrackTitleSort);
+                        break;
                     case CommonTags.Album:
                         track.AlbumTitle = Choose ((string)tag.Value, track.AlbumTitle);
                         break;
+                    case CommonTags.AlbumSortName:
+                        track.AlbumTitleSort = Choose ((string)tag.Value, track.AlbumTitleSort);
+                        break;
                     case CommonTags.Disc:
                     case CommonTags.AlbumDiscNumber:
                         int disc = (int)tag.Value;
