@@ -49,21 +49,20 @@ namespace Banshee.Collection.Database
             get { return provider; }
         }
 
-        private static HyenaSqliteCommand select_command = new HyenaSqliteCommand (String.Format (
-            "SELECT {0} FROM {1} WHERE {2} AND CoreAlbums.ArtistID = ? AND CoreAlbums.Title = ? AND CoreAlbums.IsCompilation = ?",
+        private static HyenaSqliteCommand default_select_command = new HyenaSqliteCommand (String.Format (
+            "SELECT {0} FROM {1} WHERE {2} AND CoreAlbums.ArtistID = ? AND CoreAlbums.Title = ?",
             provider.Select, provider.From,
             (String.IsNullOrEmpty (provider.Where) ? "1=1" : provider.Where)
         ));
 
-        private enum Column : int {
-            AlbumID,
-            Title,
-            ArtistName
-        }
+        private static HyenaSqliteCommand null_select_command = new HyenaSqliteCommand (String.Format (
+            "SELECT {0} FROM {1} WHERE {2} AND CoreAlbums.ArtistID = ? AND CoreAlbums.Title IS NULL",
+            provider.Select, provider.From,
+            (String.IsNullOrEmpty (provider.Where) ? "1=1" : provider.Where)
+        ));
 
         private static int last_artist_id;
         private static string last_title;
-        private static bool last_compilation;
         private static DatabaseAlbumInfo last_album;
         
         public static void Reset ()
@@ -73,35 +72,70 @@ namespace Banshee.Collection.Database
             last_album = null;
         }
 
-        public static DatabaseAlbumInfo FindOrCreate (DatabaseArtistInfo artist, string title, bool isCompilation)
+        public static DatabaseAlbumInfo FindOrCreate (DatabaseArtistInfo artist, string title, string title_sort, bool isCompilation)
         {
             DatabaseAlbumInfo album = new DatabaseAlbumInfo ();
             album.Title = title;
+            album.TitleSort = title_sort;
             album.IsCompilation = isCompilation;
             return FindOrCreate (artist, album);
         }
 
+        private static IDataReader FindExistingArtists (int artist_id, string title)
+        {
+            HyenaSqliteConnection db = ServiceManager.DbConnection;
+            if (title == null) {
+                return db.Query (null_select_command, artist_id);
+            }
+            return db.Query (default_select_command, artist_id, title);
+        }
+        
         public static DatabaseAlbumInfo FindOrCreate (DatabaseArtistInfo artist, DatabaseAlbumInfo album)
         {
-            if (album.Title == last_title && artist.DbId == last_artist_id && last_album != null && last_compilation == album.IsCompilation) {
+            if (album.Title == last_title && artist.DbId == last_artist_id && last_album != null) {
                 return last_album;
             }
 
             if (String.IsNullOrEmpty (album.Title) || album.Title.Trim () == String.Empty) {
-                album.Title = Catalog.GetString ("Unknown Album");
+                album.Title = null;
             }
 
-            using (IDataReader reader = ServiceManager.DbConnection.Query (select_command, artist.DbId, album.Title, album.IsCompilation)) {
+            using (IDataReader reader = FindExistingArtists (artist.DbId, album.Title)) {
                 if (reader.Read ()) {
+                    bool save = false;
                     last_album = provider.Load (reader);
+
                     // If the artist name has changed since last time (but it's the same artist) then update our copy of the ArtistName
                     if (last_album.ArtistName != artist.Name) {
                         last_album.ArtistName = artist.Name;
+                        save = true;
+                    }
+                    
+                    // Ditto artist sort name
+                    if (last_album.ArtistNameSort != artist.NameSort) {
+                        last_album.ArtistNameSort = artist.NameSort;
+                        save = true;
+                    }
+
+                    // And album sort name
+                    if (last_album.TitleSort != album.TitleSort) {
+                        last_album.TitleSort = album.TitleSort;
+                        save = true;
+                    }
+
+                    // If the album IsCompilation status has changed, update the saved album info
+                    if (last_album.IsCompilation != album.IsCompilation) {
+                        last_album.IsCompilation = album.IsCompilation;
+                        save = true;
+                    }
+
+                    if (save) {
                         last_album.Save ();
                     }
                 } else {
                     album.ArtistId = artist.DbId;
                     album.ArtistName = artist.Name;
+                    album.ArtistNameSort = artist.NameSort;
                     album.Save ();
                     last_album = album;
                 }
@@ -109,7 +143,6 @@ namespace Banshee.Collection.Database
             
             last_title = album.Title;
             last_artist_id = artist.DbId;
-            last_compilation = album.IsCompilation;
             return last_album;
         }
 
@@ -119,7 +152,9 @@ namespace Banshee.Collection.Database
             if (found != album) {
                 // Overwrite the found album
                 album.Title = found.Title;
+                album.TitleSort = found.TitleSort;
                 album.ArtistName = found.ArtistName;
+                album.ArtistNameSort = found.ArtistNameSort;
                 album.IsCompilation = found.IsCompilation;
                 album.dbid = found.DbId;
                 album.ArtistId = found.ArtistId;
@@ -174,9 +209,20 @@ namespace Banshee.Collection.Database
             set { base.Title = value; }
         }
         
+        [DatabaseColumn]
+        public override string TitleSort {
+            get { return base.TitleSort; }
+            set { base.TitleSort = value; }
+        }
+        
         [DatabaseColumn(Select = false)]
-        protected string TitleLowered {
-            get { return Title == null ? null : Title.ToLower (); }
+        internal byte[] TitleSortKey {
+            get { return Hyena.StringUtil.SortKey (TitleSort ?? DisplayTitle); }
+        }
+
+        [DatabaseColumn(Select = false)]
+        internal string TitleLowered {
+            get { return Hyena.StringUtil.SearchKey (DisplayTitle); }
         }
 
         [DatabaseColumn]
@@ -185,9 +231,20 @@ namespace Banshee.Collection.Database
             set { base.ArtistName = value; }
         }
 
+        [DatabaseColumn]
+        public override string ArtistNameSort {
+            get { return base.ArtistNameSort; }
+            set { base.ArtistNameSort = value; }
+        }
+
         [DatabaseColumn(Select = false)]
-        protected string ArtistNameLowered {
-            get { return ArtistName == null ? null : ArtistName.ToLower (); }
+        internal byte[] ArtistNameSortKey {
+            get { return Hyena.StringUtil.SortKey (ArtistNameSort ?? DisplayArtistName); }
+        }
+        
+        [DatabaseColumn(Select = false)]
+        internal string ArtistNameLowered {
+            get { return Hyena.StringUtil.SearchKey (DisplayArtistName); }
         }
 
         public override string ToString ()
