@@ -64,7 +64,8 @@ namespace Banshee.Collection.Database
             "ram",  "raw",  "rm",   "rmvb", "rts",  "smil", "swf",  "tivo", 
             "u",    "vfw",  "vob",  "wav",  "wave", "wax",  "wm",   "wma",  
             "wmd",  "wmv",  "wmx",  "wv",   "wvc",  "wvx",  "yuv",  "f4v",  
-            "f4a",  "f4b"
+            "f4a",  "f4b",  "669",  "it",   "med",  "mod",  "mol",  "mtm",
+            "nst",  "s3m",  "stm",  "ult",  "wow",  "xm",   "xnm",  "spx"
         );
 
         public static bool IsWhiteListedFile (string path)
@@ -128,6 +129,8 @@ namespace Banshee.Collection.Database
                 if (track != null && track.TrackId > 0) {
                     UpdateProgress (String.Format ("{0} - {1}", 
                         track.DisplayArtistName, track.DisplayTrackTitle));
+                } else {
+                    UpdateProgress (null);
                 }
                 
                 OnImportResult (track, path, null);
@@ -154,14 +157,12 @@ namespace Banshee.Collection.Database
         public DatabaseTrackInfo ImportTrack (SafeUri uri)
         {
             if (!IsWhiteListedFile (uri.LocalPath)) {
-                UpdateProgress (null);
                 return null;
             }
 
-            if (DatabaseTrackInfo.ContainsUri (uri, Paths.MakePathRelative (uri.AbsolutePath, BaseDirectory) ?? uri.AbsoluteUri, PrimarySourceIds)) {
+            if (DatabaseTrackInfo.ContainsUri (uri, PrimarySourceIds)) {
                 // TODO add DatabaseTrackInfo.SyncedStamp property, and if the file has been
                 // updated since the last sync, fetch its metadata into the db.
-                UpdateProgress (null);
                 return null;
             }
 
@@ -169,7 +170,7 @@ namespace Banshee.Collection.Database
 
             // TODO note, there is deadlock potential here b/c of locking of shared commands and blocking
             // because of transactions.  Needs to be fixed in HyenaDatabaseConnection.
-            //ServiceManager.DbConnection.BeginTransaction ();
+            ServiceManager.DbConnection.BeginTransaction ();
             try {
                 track = new DatabaseTrackInfo ();
                 track.Uri = uri;
@@ -177,15 +178,18 @@ namespace Banshee.Collection.Database
                 
                 track.PrimarySource = trackPrimarySourceChooser (track);
 
+                bool save_track = true;
                 if (track.PrimarySource is Banshee.Library.LibrarySource) {
-                    track.CopyToLibraryIfAppropriate (force_copy);
+                    save_track = track.CopyToLibraryIfAppropriate (force_copy);
                 }
 
-                track.Save (false);
+                if (save_track) {
+                    track.Save (false);
+                }
 
-                //ServiceManager.DbConnection.CommitTransaction ();
+                ServiceManager.DbConnection.CommitTransaction ();
             } catch (Exception) {
-                //ServiceManager.DbConnection.RollbackTransaction ();
+                ServiceManager.DbConnection.RollbackTransaction ();
                 throw;
             }
 
@@ -211,7 +215,7 @@ namespace Banshee.Collection.Database
 
         private void LogError (string path, string msg)
         {
-            ErrorSource.AddMessage (Path.GetFileName (path), msg);
+            ErrorSource.AddMessage (path, msg);
             Log.Error (path, msg, false);
         }
         
