@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Collections.Generic;
 using System.Data;
 using System.IO;
 
@@ -52,12 +53,6 @@ using Banshee.Streaming;
 
 namespace Banshee.Collection.Database
 {
-    public enum TrackUriType : int {
-        AbsolutePath = 0,
-        RelativePath = 1,
-        AbsoluteUri = 2
-    }
-
     public class DatabaseTrackInfo : TrackInfo
     {
         private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider;
@@ -67,7 +62,6 @@ namespace Banshee.Collection.Database
         }
 
         private bool artist_changed = false, album_changed = false;
-        private bool uri_fields_dirty = false;
         
         public DatabaseTrackInfo () : base ()
         {
@@ -78,19 +72,12 @@ namespace Banshee.Collection.Database
             Provider.Copy (original, this);
         }
 
-        public override void IncrementPlayCount ()
+        public override void OnPlaybackFinished (double percentCompleted)
         {
-            if (ProviderRefresh ()) {
-                base.IncrementPlayCount ();
-                Save (true, BansheeQuery.PlayCountField, BansheeQuery.LastPlayedField);
-            }
-        }
-
-        public override void IncrementSkipCount ()
-        {
-            if (ProviderRefresh ()) {
-                base.IncrementSkipCount ();
-                Save (true, BansheeQuery.SkipCountField, BansheeQuery.LastSkippedField);
+            if (ProviderRefresh()) {
+                base.OnPlaybackFinished (percentCompleted);
+                Save (true, BansheeQuery.ScoreField, BansheeQuery.SkipCountField, BansheeQuery.LastSkippedField, 
+                    BansheeQuery.PlayCountField, BansheeQuery.LastPlayedField);
             }
         }
 
@@ -118,18 +105,15 @@ namespace Banshee.Collection.Database
 
         public static bool TrackEqual (DatabaseTrackInfo a, DatabaseTrackInfo b)
         {
-            return a != null && b != null && 
-                a.TrackId == b.TrackId && 
-                a.CacheModelId == b.CacheModelId && 
-                (int)a.CacheEntryId == (int)b.CacheEntryId;
+            return a != null && b != null && a.TrackId == b.TrackId;
         }
         
         public DatabaseArtistInfo Artist {
-            get { return DatabaseArtistInfo.FindOrCreate (ArtistName); }
+            get { return DatabaseArtistInfo.FindOrCreate (ArtistName, ArtistNameSort); }
         }
 
         public DatabaseAlbumInfo Album {
-            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist), AlbumTitle, IsCompilation); }
+            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist, AlbumArtistSort), AlbumTitle, AlbumTitleSort, IsCompilation); }
         }
 
         private static bool notify_saved = true;
@@ -143,6 +127,16 @@ namespace Banshee.Collection.Database
             Save (NotifySaved);
         }
 
+        // Changing these fields shouldn't change DateUpdated (which triggers file save)
+        private static HashSet<QueryField> transient_fields = new HashSet<QueryField> {
+            BansheeQuery.ScoreField,
+            BansheeQuery.SkipCountField,
+            BansheeQuery.LastSkippedField,
+            BansheeQuery.PlayCountField,
+            BansheeQuery.LastPlayedField,
+            BansheeQuery.RatingField
+        };
+
         public void Save (bool notify, params QueryField [] fields_changed)
         {
             // If either the artist or album changed, 
@@ -156,10 +150,14 @@ namespace Banshee.Collection.Database
                 // TODO get rid of unused artists/albums
             }
             
-            DateUpdated = DateTime.Now;
+            if (fields_changed.Length == 0 || !transient_fields.IsSupersetOf (fields_changed)) {
+                DateUpdated = DateTime.Now;
+            }
 
             bool is_new = (TrackId == 0);
-            if (is_new) DateAdded = DateUpdated;
+            if (is_new) {
+                DateAdded = DateUpdated = DateTime.Now;
+            }
 
             ProviderSave ();
 
@@ -198,10 +196,7 @@ namespace Banshee.Collection.Database
         [DatabaseColumn ("PrimarySourceID")]
         public int PrimarySourceId {
             get { return primary_source_id; }
-            set {
-                primary_source_id = value;
-                UpdateUri ();
-            }
+            set { primary_source_id = value; }
         }
 
         public PrimarySource PrimarySource {
@@ -241,6 +236,24 @@ namespace Banshee.Collection.Database
             }
         }
 
+        [VirtualDatabaseColumn ("NameSort", "CoreArtists", "ArtistID", "ArtistID")]
+        protected string ArtistNameSortField {
+            get { return ArtistNameSort; }
+            set { base.ArtistNameSort = value; }
+        }
+
+        public override string ArtistNameSort {
+            get { return base.ArtistNameSort; }
+            set {
+                value = CleanseString (value, ArtistNameSort);
+                if (value == null)
+                    return;
+
+                base.ArtistNameSort = value;
+                artist_changed = true;
+            }
+        }
+
         [VirtualDatabaseColumn ("Title", "CoreAlbums", "AlbumID", "AlbumID")]
         protected string AlbumTitleField {
             get { return AlbumTitle; }
@@ -259,6 +272,24 @@ namespace Banshee.Collection.Database
             }
         }
 
+        [VirtualDatabaseColumn ("TitleSort", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected string AlbumTitleSortField {
+            get { return AlbumTitleSort; }
+            set { base.AlbumTitleSort = value; }
+        }
+
+        public override string AlbumTitleSort {
+            get { return base.AlbumTitleSort; }
+            set {
+                value = CleanseString (value, AlbumTitleSort);
+                if (value == null)
+                    return;
+
+                base.AlbumTitleSort = value;
+                album_changed = true;
+            }
+        }
+
         [VirtualDatabaseColumn ("ArtistName", "CoreAlbums", "AlbumID", "AlbumID")]
         protected string AlbumArtistField {
             get { return AlbumArtist; }
@@ -277,6 +308,24 @@ namespace Banshee.Collection.Database
             }
         }
         
+        [VirtualDatabaseColumn ("ArtistNameSort", "CoreAlbums", "AlbumID", "AlbumID")]
+        protected string AlbumArtistSortField {
+            get { return AlbumArtistSort; }
+            set { base.AlbumArtistSort = value; }
+        }
+
+        public override string AlbumArtistSort {
+            get { return base.AlbumArtistSort; }
+            set {
+                value = CleanseString (value, AlbumArtistSort);
+                if (value == null)
+                    return;
+
+                base.AlbumArtistSort = value;
+                album_changed = true;
+            }
+        }
+        
         [VirtualDatabaseColumn ("IsCompilation", "CoreAlbums", "AlbumID", "AlbumID")]
         protected bool IsCompilationField {
             get { return IsCompilation; }
@@ -318,46 +367,10 @@ namespace Banshee.Collection.Database
             set { base.MusicBrainzId = value; }
         }
 
-        public override SafeUri Uri {
-            get { return base.Uri; }
-            set {
-                base.Uri = value;
-                uri_fields_dirty = true;
-            }
-        }
-        
-        private string uri_field;
         [DatabaseColumn ("Uri")]
         protected string UriField {
-            get {
-                if (uri_fields_dirty && Uri != null && PrimarySource != null) {
-                    PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
-                    uri_fields_dirty = false;
-                }
-                return uri_field;
-            }
-            set {
-                uri_field = value;
-                UpdateUri ();
-            }
-        }
-        
-        private bool uri_type_set;
-        private TrackUriType uri_type;
-        [DatabaseColumn ("UriType")]
-        protected TrackUriType UriType {
-            get {
-                if (uri_fields_dirty && Uri != null && PrimarySource != null) {
-                    PrimarySource.UriToFields (Uri, out uri_type, out uri_field);
-                    uri_fields_dirty = false;
-                }
-                return uri_type;
-            }
-            set {
-                uri_type = value;
-                uri_type_set = true;
-                UpdateUri ();
-            }
+            get { return Uri.AbsoluteUri; }
+            set { Uri = new SafeUri (value); }
         }
         
         [DatabaseColumn]
@@ -396,9 +409,20 @@ namespace Banshee.Collection.Database
             set { base.TrackTitle = value; }
         }
         
+        [DatabaseColumn ("TitleSort")]
+        public override string TrackTitleSort {
+            get { return base.TrackTitleSort; }
+            set { base.TrackTitleSort = value; }
+        }
+        
+        [DatabaseColumn("TitleSortKey", Select = false)]
+        internal byte[] TrackTitleSortKey {
+            get { return Hyena.StringUtil.SortKey (TrackTitleSort ?? DisplayTrackTitle); }
+        }
+        
         [DatabaseColumn(Select = false)]
-        protected string TitleLowered {
-            get { return TrackTitle == null ? null : TrackTitle.ToLower (); }
+        internal string TitleLowered {
+            get { return Hyena.StringUtil.SearchKey (DisplayTrackTitle); }
         }
 
         [DatabaseColumn(Select = false)]
@@ -503,6 +527,12 @@ namespace Banshee.Collection.Database
             set { rating = value; }
         }
 
+        [DatabaseColumn]
+        public override int Score {
+            get { return base.Score; }
+            set { base.Score = value; }
+        }
+
         public int SavedRating {
             get { return rating; }
             set {
@@ -580,68 +610,74 @@ namespace Banshee.Collection.Database
             }
         }
 
-        private void UpdateUri ()
+        public bool CopyToLibraryIfAppropriate (bool force_copy)
         {
-            if (Uri == null && uri_type_set && UriField != null && PrimarySource != null) {
-                Uri = PrimarySource.UriAndTypeToSafeUri (UriType, UriField);
-            }
-        }
+            bool copy_success = true;
 
-        public void CopyToLibraryIfAppropriate (bool force_copy)
-        {
             SafeUri old_uri = this.Uri;
             if (old_uri == null) {
                 // Get out quick, no URI set yet.
-                return;
+                return copy_success;
             }
             
-            bool in_library = old_uri.AbsolutePath.StartsWith (Paths.CachedLibraryLocationWithSeparator);
+            bool in_library = old_uri.AbsolutePath.StartsWith (PrimarySource.BaseDirectoryWithSeparator);
 
             if (!in_library && (LibrarySchema.CopyOnImport.Get () || force_copy)) {
-                string new_filename = FileNamePattern.BuildFull (this, Path.GetExtension (old_uri.ToString ()));
+                string new_filename = FileNamePattern.BuildFull (PrimarySource.BaseDirectory, this, Path.GetExtension (old_uri.ToString ()));
                 SafeUri new_uri = new SafeUri (new_filename);
 
                 try {
                     if (Banshee.IO.File.Exists (new_uri)) {
-                        return;
+                        if (Banshee.IO.File.GetSize (old_uri) == Banshee.IO.File.GetSize (new_uri)) {
+                            Hyena.Log.DebugFormat ("Not copying {0} to library because there is already a file of same size at {1}", old_uri, new_uri);
+                            copy_success = false;
+                            return copy_success;
+                        } else {
+                            string extension = Path.GetExtension (new_filename);
+                            string filename_no_ext = new_filename.Remove (new_filename.Length - extension.Length);
+                            int duplicate_index = 1;
+                            while (Banshee.IO.File.Exists (new_uri)) {
+                                new_filename = String.Format ("{0} ({1}){2}", filename_no_ext, duplicate_index, extension);
+                                new_uri = new SafeUri (new_filename);
+                                duplicate_index++;
+                          }
+                        }
                     }
-                    
+
                     Banshee.IO.File.Copy (old_uri, new_uri, false);
                     Uri = new_uri;
                 } catch (Exception e) {
                     Log.ErrorFormat ("Exception copying into library: {0}", e);
                 }
             }
+            return copy_success;
         }
 
-        private static HyenaSqliteCommand check_command = new HyenaSqliteCommand (
-            "SELECT TrackID FROM CoreTracks WHERE PrimarySourceId IN (?) AND (Uri = ? OR Uri = ?) LIMIT 1"
+        private static HyenaSqliteCommand get_uri_id_cmd = new HyenaSqliteCommand ("SELECT TrackID FROM CoreTracks WHERE Uri = ? LIMIT 1");
+        public static int GetTrackIdForUri (string uri)
+        {
+            return ServiceManager.DbConnection.Query<int> (get_uri_id_cmd, new SafeUri (uri).AbsoluteUri);
+        }
+
+        private static HyenaSqliteCommand get_track_id_by_uri = new HyenaSqliteCommand (
+            "SELECT TrackID FROM CoreTracks WHERE PrimarySourceId IN (?) AND Uri = ? LIMIT 1"
         );
 
-        public static int GetTrackIdForUri (SafeUri uri, string relative_path, int [] primary_sources)
+        public static int GetTrackIdForUri (SafeUri uri, int [] primary_sources)
         {
-            return ServiceManager.DbConnection.Query<int> (check_command,
-                primary_sources, relative_path, uri.AbsoluteUri);
+            return ServiceManager.DbConnection.Query<int> (get_track_id_by_uri,
+                primary_sources, uri.AbsoluteUri);
         }
 
-        public static int GetTrackIdForUri (string relative_path, int [] primary_sources)
+        public static int GetTrackIdForUri (string absoluteUri, int [] primary_sources)
         {
-            return GetTrackIdForUri (relative_path, relative_path, primary_sources);
-        }
-        
-        public static int GetTrackIdForUri (string uri, string relative_path, int [] primary_sources)
-        {
-            return ServiceManager.DbConnection.Query<int> (check_command, primary_sources, uri, relative_path);
-        }
-        
-        public static bool ContainsUri (string relative_path, int [] primary_sources)
-        {
-            return GetTrackIdForUri (relative_path, primary_sources) > 0;
+            return ServiceManager.DbConnection.Query<int> (get_track_id_by_uri,
+                primary_sources, absoluteUri);
         }
 
-        public static bool ContainsUri (SafeUri uri, string relative_path, int [] primary_sources)
+        public static bool ContainsUri (SafeUri uri, int [] primary_sources)
         {
-            return GetTrackIdForUri (uri, relative_path, primary_sources) > 0;
+            return GetTrackIdForUri (uri, primary_sources) > 0;
         }
     }
 }
