@@ -31,6 +31,7 @@ using System;
 using System.Data;
 using System.Text;
 using System.Collections.Generic;
+using System.Linq;
 
 using Mono.Unix;
 
@@ -42,6 +43,7 @@ using Hyena.Query;
 using Banshee.Base;
 using Banshee.Query;
 using Banshee.Database;
+using Banshee.PlaybackController;
 
 namespace Banshee.Collection.Database
 {       
@@ -64,7 +66,7 @@ namespace Banshee.Collection.Database
         private bool forced_sort_query;
         
         private string reload_fragment;
-        private string join_table, join_fragment, join_primary_key, join_column, condition;
+        private string join_table, join_fragment, join_primary_key, join_column, condition, condition_from;
 
         private string query_fragment;
         private string user_query;
@@ -185,11 +187,11 @@ namespace Banshee.Collection.Database
         }
 
         private string unfiltered_query;
-        public string UnfilteredQuery {
+        public virtual string UnfilteredQuery {
             get {
                 return unfiltered_query ?? (unfiltered_query = String.Format (
-                    "FROM {0}{1} WHERE {2} {3}",
-                    From, JoinFragment,
+                    "FROM {0} WHERE {1} {2}",
+                    FromFragment,
                     String.IsNullOrEmpty (provider.Where) ? "1=1" : provider.Where,
                     ConditionFragment
                 ));
@@ -200,7 +202,12 @@ namespace Banshee.Collection.Database
         protected string From {
             get { return from ?? provider.From; }
             set { from = value; }
-        }     
+        }
+
+        private string from_fragment;
+        public string FromFragment {
+            get { return from_fragment ?? (from_fragment = String.Format ("{0}{1}", From, JoinFragment)); }
+        }
 
         public virtual void UpdateUnfilteredAggregates ()
         {
@@ -331,23 +338,30 @@ namespace Banshee.Collection.Database
             }
         }
 
+#region Get random methods
+
+        private static RandomBy [] randoms = new RandomBy [] {
+            new RandomByTrack (), new RandomByArtist (), new RandomByAlbum (), new RandomByRating (), new RandomByScore()
+        };
+
         private DateTime random_began_at = DateTime.MinValue;
         private DateTime last_random = DateTime.MinValue;
-        private static string random_fragment = "AND (LastPlayedStamp < ? OR LastPlayedStamp IS NULL) AND (LastSkippedStamp < ? OR LastSkippedStamp IS NULL) ORDER BY RANDOM()";
-        public override TrackInfo GetRandom (DateTime notPlayedSince, bool repeat)
+
+        public override TrackInfo GetRandom (DateTime notPlayedSince, PlaybackShuffleMode mode, bool repeat, bool lastWasSkipped)
         {
             lock (this) {
-                if (Count == 0)
+                if (Count == 0) {
                     return null;
+                }
 
-                if (random_began_at < notPlayedSince)
+                if (random_began_at < notPlayedSince) {
                     random_began_at = last_random = notPlayedSince;
+                }
 
-                TrackInfo track = cache.GetSingle (random_fragment, random_began_at, random_began_at);
-
-                if (track == null && repeat) {
-                    random_began_at = last_random;
-                    track = cache.GetSingle (random_fragment, random_began_at, random_began_at);
+                TrackInfo track = GetRandomTrack (mode, repeat, lastWasSkipped);
+                if (track == null && (repeat || mode != PlaybackShuffleMode.Linear)) {
+                    random_began_at = (random_began_at == last_random) ? DateTime.Now : last_random;
+                    track = GetRandomTrack (mode, repeat, true);
                 }
 
                 last_random = DateTime.Now;
@@ -355,6 +369,34 @@ namespace Banshee.Collection.Database
             }
         }
 
+        private TrackInfo GetRandomTrack (PlaybackShuffleMode mode, bool repeat, bool lastWasSkipped)
+        {
+            foreach (var r in randoms) {
+                r.SetModelAndCache (this, cache);
+                if (lastWasSkipped || r.Mode != mode) {
+                    r.Reset ();
+                }
+            }
+            
+            var random = randoms.First (r => r.Mode == mode);
+            if (random != null) {
+                if (!random.IsReady) {
+                    if (!random.Next (random_began_at) && repeat) {
+                        random_began_at = last_random;
+                        random.Next (random_began_at);
+                    }
+                }
+
+                if (random.IsReady) {
+                    return random.GetTrack (random_began_at);
+                }
+            }
+
+            return null;
+        }
+
+#endregion
+
         public override TrackInfo this[int index] {
             get {
                 lock (this) {
@@ -426,11 +468,20 @@ namespace Banshee.Collection.Database
             get { return join_column; }
             set { join_column = value; }
         }
-        
+
         public void AddCondition (string part)
+        {
+            AddCondition (null, part);
+        }
+        
+        public void AddCondition (string tables, string part)
         {
             if (!String.IsNullOrEmpty (part)) {
                 condition = condition == null ? part : String.Format ("{0} AND {1}", condition, part);
+
+                if (!String.IsNullOrEmpty (tables)) {
+                    condition_from = condition_from == null ? tables : String.Format ("{0}, {1}", condition_from, tables);
+                }
             }
         }
         
@@ -438,6 +489,25 @@ namespace Banshee.Collection.Database
             get { return condition; }
         }
 
+        private string condition_from_fragment;
+        public string ConditionFromFragment {
+            get {
+                if (condition_from_fragment == null) {
+                    if (JoinFragment == null) {
+                        condition_from_fragment = condition_from;
+                    } else {
+                        if (condition_from == null) {
+                            condition_from = "CoreTracks";
+                        }
+
+                        condition_from_fragment = String.Format ("{0}{1}", condition_from, JoinFragment);
+                    }
+                }
+
+                return condition_from_fragment;
+            }
+        }
+
         public string ConditionFragment {
             get { return PrefixCondition ("AND"); }
         }
