@@ -3,8 +3,9 @@
 //
 // Author:
 //   Aaron Bockover <abockover@novell.com>
+//   Gabriel Burt <gburt@novell.com>
 //
-// Copyright (C) 2007-2008 Novell, Inc.
+// Copyright (C) 2007-2009 Novell, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,10 +30,13 @@
 using System;
 using System.Data;
 using System.Reflection;
+using System.Text;
 using System.Threading;
 using Mono.Unix;
+using Mono.Data.Sqlite;
 
 using Hyena;
+using Hyena.Jobs;
 using Hyena.Data.Sqlite;
 using Timer=Hyena.Timer;
 
@@ -52,8 +56,8 @@ namespace Banshee.Database
         // NOTE: Whenever there is a change in ANY of the database schema,
         //       this version MUST be incremented and a migration method
         //       MUST be supplied to match the new version number
-        protected const int CURRENT_VERSION = 22;
-        protected const int CURRENT_METADATA_VERSION = 4;
+        protected const int CURRENT_VERSION = 35;
+        protected const int CURRENT_METADATA_VERSION = 6;
         
 #region Migration Driver
         
@@ -227,6 +231,7 @@ namespace Banshee.Database
                     Catalog.GetString ("Please wait while your old Banshee database is migrated to the new format."));
             
                 Thread thread = new Thread (MigrateFromLegacyBanshee);
+                thread.Name = "Database Migrator";
                 thread.Start ();
             
                 while (thread.IsAlive) {
@@ -333,7 +338,7 @@ namespace Banshee.Database
         [DatabaseVersion (6)]
         private bool Migrate_6 ()
         {
-            Execute ("INSERT INTO CoreConfiguration VALUES (null, 'MetadataVersion', 0)");
+            Execute ("INSERT INTO CoreConfiguration (EntryID, Key, Value) VALUES (null, 'MetadataVersion', 0)");
             return true;
         }
         
@@ -344,10 +349,10 @@ namespace Banshee.Database
         [DatabaseVersion (7)]
         private bool Migrate_7 ()
         {
-            Execute ("UPDATE CorePrimarySources SET StringID = 'MusicLibrarySource-Library' WHERE StringID = 'Library'");
-            Execute ("UPDATE CorePrimarySources SET StringID = 'VideoLibrarySource-VideoLibrary' WHERE StringID = 'VideoLibrary'");
-            Execute ("UPDATE CorePrimarySources SET StringID = 'PodcastSource-podcasting' WHERE StringID = 'podcasting'");
-            Execute ("DELETE FROM CoreCache; DELETE FROM CoreCacheModels");
+            try { Execute ("UPDATE CorePrimarySources SET StringID = 'MusicLibrarySource-Library' WHERE StringID = 'Library'"); } catch {}
+            try { Execute ("UPDATE CorePrimarySources SET StringID = 'VideoLibrarySource-VideoLibrary' WHERE StringID = 'VideoLibrary'"); } catch {}
+            try { Execute ("UPDATE CorePrimarySources SET StringID = 'PodcastSource-podcasting' WHERE StringID = 'podcasting'"); } catch {}
+            try { Execute ("DELETE FROM CoreCache; DELETE FROM CoreCacheModels"); } catch {}
             return true;
         }
         
@@ -550,6 +555,253 @@ namespace Banshee.Database
         
 #endregion
 
+#region Version 23
+
+        [DatabaseVersion (23)]
+        private bool Migrate_23 ()
+        {
+            Execute ("ALTER TABLE CoreArtists ADD COLUMN NameSort TEXT");
+            Execute ("ALTER TABLE CoreAlbums ADD COLUMN ArtistNameSort TEXT");
+            Execute ("ALTER TABLE CoreAlbums ADD COLUMN TitleSort TEXT");
+            Execute ("ALTER TABLE CoreTracks ADD COLUMN TitleSort TEXT");
+
+            return true;
+        }
+        
+#endregion
+
+#region Version 24
+        [DatabaseVersion (24)]
+        private bool Migrate_24 ()
+        {
+            Execute ("UPDATE CoreArtists SET NameLowered = HYENA_SEARCH_KEY(Name)");
+            Execute ("UPDATE CoreAlbums SET ArtistNameLowered = HYENA_SEARCH_KEY(ArtistName)");
+            Execute ("UPDATE CoreAlbums SET TitleLowered = HYENA_SEARCH_KEY(Title)");
+            Execute ("UPDATE CoreTracks SET TitleLowered = HYENA_SEARCH_KEY(Title)");
+            return true;
+        }
+#endregion
+
+#region Version 25
+
+        [DatabaseVersion (25)]
+        private bool Migrate_25 ()
+        {
+            Execute ("ALTER TABLE CoreArtists ADD COLUMN NameSortKey BLOB");
+            Execute ("ALTER TABLE CoreAlbums ADD COLUMN ArtistNameSortKey BLOB");
+            Execute ("ALTER TABLE CoreAlbums ADD COLUMN TitleSortKey BLOB");
+            Execute ("ALTER TABLE CoreTracks ADD COLUMN TitleSortKey BLOB");
+            return true;
+        }
+        
+#endregion
+
+#region Version 26
+
+        [DatabaseVersion (26)]
+        private bool Migrate_26 ()
+        {
+            string unknown_artist = "Unknown Artist";
+            string unknown_album = "Unknown Album";
+            string unknown_title = "Unknown Title";
+
+            connection.Execute ("UPDATE CoreArtists SET Name = NULL, NameLowered = HYENA_SEARCH_KEY(?)" +
+                                " WHERE Name  IN ('', ?, ?) OR Name IS NULL", 
+                                ArtistInfo.UnknownArtistName, unknown_artist, ArtistInfo.UnknownArtistName);
+
+            connection.Execute ("UPDATE CoreAlbums SET ArtistName = NULL, ArtistNameLowered = HYENA_SEARCH_KEY(?)" +
+                                " WHERE ArtistName IN ('', ?, ?) OR ArtistName IS NULL",
+                                ArtistInfo.UnknownArtistName, unknown_artist, ArtistInfo.UnknownArtistName);
+
+            connection.Execute ("UPDATE CoreAlbums SET Title = NULL, TitleLowered = HYENA_SEARCH_KEY(?)" +
+                                " WHERE Title IN ('', ?, ?) OR Title IS NULL",
+                                AlbumInfo.UnknownAlbumTitle, unknown_album, AlbumInfo.UnknownAlbumTitle);
+
+            connection.Execute ("UPDATE CoreTracks SET Title = NULL, TitleLowered = HYENA_SEARCH_KEY(?)" +
+                                " WHERE Title IN ('', ?, ?) OR Title IS NULL",
+                                TrackInfo.UnknownTitle, unknown_title, TrackInfo.UnknownTitle);
+
+            return true;
+        }
+        
+#endregion
+
+#region Version 27
+
+        [DatabaseVersion (27)]
+        private bool Migrate_27 ()
+        {
+            // One time fixup to MetadataHash now that our unknown metadata is handled properly
+            string sql_select = @"
+                SELECT t.TrackID, al.Title, ar.Name, t.Duration,
+                t.Genre, t.Title, t.TrackNumber, t.Year
+                FROM CoreTracks AS t
+                JOIN CoreAlbums AS al ON al.AlbumID=t.AlbumID
+                JOIN CoreArtists AS ar ON ar.ArtistID=t.ArtistID
+                WHERE t.Title IS NULL OR ar.Name IS NULL OR al.Title IS NULL
+            ";
+
+            HyenaSqliteCommand sql_update = new HyenaSqliteCommand (@"
+                UPDATE CoreTracks SET MetadataHash = ? WHERE TrackID = ?
+            ");
+
+            StringBuilder sb = new StringBuilder ();
+            using (var reader = new HyenaDataReader (connection.Query (sql_select))) {
+                while (reader.Read ()) {
+                    sb.Length = 0;
+                    sb.Append (reader.Get<string> (1));
+                    sb.Append (reader.Get<string> (2));
+                    sb.Append ((int)reader.Get<TimeSpan> (3).TotalSeconds);
+                    sb.Append (reader.Get<string> (4));
+                    sb.Append (reader.Get<string> (5));
+                    sb.Append (reader.Get<int> (6));
+                    sb.Append (reader.Get<int> (7));
+                    string hash = Hyena.CryptoUtil.Md5Encode (sb.ToString (), System.Text.Encoding.UTF8);
+                    connection.Execute (sql_update, hash, reader.Get<int> (0));
+                }
+            }
+
+            return true;
+        }
+
+#endregion
+
+#region Version 28
+
+        [DatabaseVersion (28)]
+        private bool Migrate_28 ()
+        {
+            // Update search keys for new space-stripping behavior.
+            connection.Execute ("UPDATE CoreArtists SET NameLowered = HYENA_SEARCH_KEY(IFNULL(Name, ?))",
+                                ArtistInfo.UnknownArtistName);
+            connection.Execute ("UPDATE CoreAlbums SET ArtistNameLowered = HYENA_SEARCH_KEY(IFNULL(ArtistName, ?))," +
+                                "                      TitleLowered = HYENA_SEARCH_KEY(IFNULL(Title, ?))",
+                                ArtistInfo.UnknownArtistName, AlbumInfo.UnknownAlbumTitle);
+            connection.Execute ("UPDATE CoreTracks SET TitleLowered = HYENA_SEARCH_KEY(IFNULL(Title, ?))",
+                                TrackInfo.UnknownTitle);
+            return true;
+        }
+
+#endregion
+
+#region Version 29
+
+        [DatabaseVersion (29)]
+        private bool Migrate_29 ()
+        {
+            Execute ("ALTER TABLE CoreTracks ADD COLUMN Score INTEGER DEFAULT 0");
+            Execute (@"
+                UPDATE CoreTracks
+                SET Score = CAST(ROUND(100.00 * PlayCount / (PlayCount + SkipCount)) AS INTEGER)
+                WHERE PlayCount + SkipCount > 0
+            ");
+            return true;
+        }
+
+#endregion
+
+#region Version 30
+
+        [DatabaseVersion (30)]
+        private bool Migrate_30 ()
+        {
+            Execute ("DROP INDEX IF EXISTS CoreAlbumsIndex");
+            Execute ("DROP INDEX IF EXISTS CoreAlbumsArtistIndex");
+            Execute ("DROP INDEX IF EXISTS CoreArtistsIndex");
+            Execute ("CREATE INDEX CoreAlbumsIndex ON CoreAlbums(ArtistID, TitleSortKey)");
+            Execute ("CREATE INDEX CoreAlbumsArtistIndex ON CoreAlbums(TitleSortKey, ArtistNameSortKey)");
+            Execute ("CREATE INDEX CoreArtistsIndex ON CoreArtists(NameSortKey)");
+            Execute ("ANALYZE");
+            return true;
+        }
+
+#endregion
+
+
+#region Version 31
+
+        [DatabaseVersion (31)]
+        private bool Migrate_31 ()
+        {
+            try {
+                // Make paths not relative for Music Library items
+                string library_path = Banshee.Library.LibrarySource.OldLocationSchema.Get (Banshee.Library.MusicLibrarySource.GetDefaultBaseDirectory ());
+                if (library_path != null) {
+                    int podcast_src_id = connection.Query<int> ("SELECT PrimarySourceID FROM CorePrimarySources WHERE StringID = 'PodcastSource-PodcastLibrary'");
+
+                    connection.Execute (@"
+                        UPDATE CoreTracks SET Uri = BANSHEE_MIGRATE_PARTIAL(?, Uri)
+                        WHERE UriType = 1
+                          AND PrimarySourceID != ?", library_path, podcast_src_id);
+
+                    string podcast_path = Banshee.Base.Paths.Combine (library_path, "Podcasts");
+                    connection.Execute (@"
+                        UPDATE CoreTracks SET Uri = BANSHEE_MIGRATE_PARTIAL(?, Uri)
+                        WHERE UriType = 1
+                          AND PrimarySourceID = ?", podcast_path, podcast_src_id);
+                }
+            } catch (Exception e) {
+                Hyena.Log.Exception (e);
+            }
+            return true;
+        }
+
+#endregion
+
+#region Version 32
+
+        [DatabaseVersion (32)]
+        private bool Migrate_32 ()
+        {
+            Execute ("DROP INDEX IF EXISTS CoreSmartPlaylistEntriesPlaylistIndex");
+            Execute ("DROP INDEX IF EXISTS CoreSmartPlaylistEntriesIndex");
+            Execute ("ANALYZE");
+            return true;
+        }
+        
+#endregion
+
+#region Version 33
+
+        [DatabaseVersion (33)]
+        private bool Migrate_33 ()
+        {
+            // We used to have a bug where MimeType didn't get set for tracks we ripped,
+            // so set any blank ones now.  See BGO #536590
+            foreach (var ext in new string [] {"mp3", "ogg", "flac", "aac", "oga", "wma", "wm"}) {
+                Execute (String.Format (
+                    "UPDATE CoreTracks SET MimeType = 'taglib/{0}' WHERE PrimarySourceId = 1 AND (MimeType IS NULL OR MimeType = '') AND Uri LIKE '%.{0}'", ext
+                ));
+            }
+            return true;
+        }
+
+#endregion
+
+#region Version 34
+
+        [DatabaseVersion (34)]
+        private bool Migrate_34 ()
+        {
+            Execute ("CREATE INDEX IF NOT EXISTS CoreSmartPlaylistEntriesIndex ON CoreSmartPlaylistEntries(SmartPlaylistID, TrackID)");
+            return true;
+        }
+
+#endregion
+
+#region Version 35
+
+        [DatabaseVersion (35)]
+        private bool Migrate_35 ()
+        {
+            if (!connection.ColumnExists ("CorePlaylistEntries", "Generated")) {
+                Execute ("ALTER TABLE CorePlaylistEntries ADD COLUMN Generated INTEGER NOT NULL DEFAULT 0");
+            }
+            return true;
+        }
+
+#endregion
+
 #pragma warning restore 0169
         
 #region Fresh database setup
@@ -575,9 +827,9 @@ namespace Banshee.Database
                     Value               TEXT
                 )
             ");
-            Execute (String.Format ("INSERT INTO CoreConfiguration VALUES (null, 'DatabaseVersion', {0})", CURRENT_VERSION));
+            Execute (String.Format ("INSERT INTO CoreConfiguration (EntryID, Key, Value) VALUES (null, 'DatabaseVersion', {0})", CURRENT_VERSION));
             if (!refresh_metadata) {
-                Execute (String.Format ("INSERT INTO CoreConfiguration VALUES (null, 'MetadataVersion', {0})", CURRENT_METADATA_VERSION));
+                Execute (String.Format ("INSERT INTO CoreConfiguration (EntryID, Key, Value) VALUES (null, 'MetadataVersion', {0})", CURRENT_METADATA_VERSION));
             }
             
             Execute(@"
@@ -605,7 +857,6 @@ namespace Banshee.Database
                     MusicBrainzID       TEXT,
 
                     Uri                 TEXT,
-                    UriType             INTEGER,
                     MimeType            TEXT,
                     FileSize            INTEGER,
                     BitRate             INTEGER,
@@ -614,6 +865,8 @@ namespace Banshee.Database
                     
                     Title               TEXT,
                     TitleLowered        TEXT,
+                    TitleSort           TEXT,
+                    TitleSortKey        BLOB,
                     TrackNumber         INTEGER,
                     TrackCount          INTEGER,
                     Disc                INTEGER,
@@ -629,6 +882,7 @@ namespace Banshee.Database
 
                     Comment             TEXT,
                     Rating              INTEGER,
+                    Score               INTEGER,
                     PlayCount           INTEGER,
                     SkipCount           INTEGER,
                     LastPlayedStamp     INTEGER,
@@ -641,6 +895,7 @@ namespace Banshee.Database
                     FileModifiedStamp   INTEGER
                 )
             ", (int)TrackMediaAttributes.Default, (int)StreamPlaybackError.None));
+
             Execute("CREATE INDEX CoreTracksPrimarySourceIndex ON CoreTracks(ArtistID, AlbumID, PrimarySourceID, Disc, TrackNumber, Uri)");
             Execute("CREATE INDEX CoreTracksAggregatesIndex ON CoreTracks(FileSize, Duration)");
             Execute("CREATE INDEX CoreTracksExternalIDIndex ON CoreTracks(PrimarySourceID, ExternalID)");
@@ -657,6 +912,8 @@ namespace Banshee.Database
 
                     Title               TEXT,
                     TitleLowered        TEXT,
+                    TitleSort           TEXT,
+                    TitleSortKey        BLOB,
 
                     ReleaseDate         INTEGER,
                     Duration            INTEGER,
@@ -665,12 +922,14 @@ namespace Banshee.Database
                     
                     ArtistName          TEXT,
                     ArtistNameLowered   TEXT,
+                    ArtistNameSort      TEXT,
+                    ArtistNameSortKey   BLOB,
                     
                     Rating              INTEGER
                 )
             ");
-            Execute("CREATE INDEX CoreAlbumsIndex       ON CoreAlbums(ArtistID, TitleLowered)");
-            Execute("CREATE INDEX CoreAlbumsArtistIndex       ON CoreAlbums(TitleLowered, ArtistNameLowered)");
+            Execute ("CREATE INDEX CoreAlbumsIndex ON CoreAlbums(ArtistID, TitleSortKey)");
+            Execute ("CREATE INDEX CoreAlbumsArtistIndex ON CoreAlbums(TitleSortKey, ArtistNameSortKey)");
 
             Execute(@"
                 CREATE TABLE CoreArtists (
@@ -679,10 +938,12 @@ namespace Banshee.Database
                     MusicBrainzID       TEXT,
                     Name                TEXT,
                     NameLowered         TEXT,
+                    NameSort            TEXT,
+                    NameSortKey         BLOB,
                     Rating              INTEGER
                 )
             ");
-            Execute("CREATE INDEX CoreArtistsIndex ON CoreArtists(NameLowered)");
+            Execute ("CREATE INDEX CoreArtistsIndex ON CoreArtists(NameSortKey)");
             
             Execute(@"
                 CREATE TABLE CorePlaylists (
@@ -702,7 +963,8 @@ namespace Banshee.Database
                     EntryID             INTEGER PRIMARY KEY,
                     PlaylistID          INTEGER NOT NULL,
                     TrackID             INTEGER NOT NULL,
-                    ViewOrder           INTEGER NOT NULL DEFAULT 0
+                    ViewOrder           INTEGER NOT NULL DEFAULT 0,
+                    Generated           INTEGER NOT NULL DEFAULT 0
                 )
             ");
             Execute("CREATE INDEX CorePlaylistEntriesIndex ON CorePlaylistEntries(PlaylistID, TrackID)");
@@ -728,7 +990,7 @@ namespace Banshee.Database
                     TrackID             INTEGER NOT NULL
                 )
             ");
-            Execute("CREATE INDEX CoreSmartPlaylistEntriesPlaylistIndex ON CoreSmartPlaylistEntries(SmartPlaylistID, TrackID)");
+            Execute ("CREATE INDEX CoreSmartPlaylistEntriesIndex ON CoreSmartPlaylistEntries(SmartPlaylistID, TrackID)");
 
             Execute(@"
                 CREATE TABLE CoreRemovedTracks (
@@ -757,26 +1019,34 @@ namespace Banshee.Database
         private void MigrateFromLegacyBanshee()
         {
             Execute(@"
-                INSERT INTO CoreArtists 
-                    SELECT DISTINCT null, 0, null, Artist, NULL, 0 
+                INSERT INTO CoreArtists
+                    (ArtistID, TagSetID, MusicBrainzID, Name, NameLowered, NameSort, Rating)
+                    SELECT DISTINCT null, 0, null, Artist, NULL, NULL, 0
                         FROM Tracks 
                         ORDER BY Artist
             ");
             
             Execute(@"
                 INSERT INTO CoreAlbums
+                    (AlbumID, ArtistID, TagSetID, MusicBrainzID, Title, TitleLowered, TitleSort, ReleaseDate,
+                    Duration, Year, IsCompilation, ArtistName, ArtistNameLowered, ArtistNameSort, Rating)
                     SELECT DISTINCT null,
                         (SELECT ArtistID 
                             FROM CoreArtists 
                             WHERE Name = Tracks.Artist
                             LIMIT 1),
-                        0, null, AlbumTitle, NULL, ReleaseDate, 0, 0, 0, Artist, NULL, 0
+                        0, null, AlbumTitle, NULL, NULL, ReleaseDate, 0, 0, 0, Artist, NULL, NULL, 0
                         FROM Tracks
                         ORDER BY AlbumTitle
             ");
             
             Execute (String.Format (@"
                 INSERT INTO CoreTracks
+                    (PrimarySourceID, TrackID, ArtistID, AlbumID, TagSetID, ExternalID, MusicBrainzID, Uri, MimeType,
+                    FileSize, BitRate, Attributes, LastStreamError, Title, TitleLowered, TrackNumber, TrackCount,
+                    Disc, DiscCount, Duration, Year, Genre, Composer, Conductor, Grouping, Copyright, LicenseUri,
+                    Comment, Rating, Score, PlayCount, SkipCount, LastPlayedStamp, LastSkippedStamp, DateAddedStamp,
+                    DateUpdatedStamp, MetadataHash, BPM, LastSyncedStamp, FileModifiedStamp)
                     SELECT 
                         1,
                         TrackID, 
@@ -792,7 +1062,6 @@ namespace Banshee.Database
                         0,
                         0,
                         Uri,
-                        0,
                         MimeType,
                         0, 0,
                         {0},
@@ -806,6 +1075,7 @@ namespace Banshee.Database
                         Genre,
                         NULL, NULL, NULL, NULL, NULL, NULL,
                         Rating,
+                        0,
                         NumberOfPlays,
                         0,
                         LastPlayedStamp,
@@ -824,6 +1094,7 @@ namespace Banshee.Database
                     INSERT INTO CorePlaylists (PlaylistID, Name, SortColumn, SortType)
                         SELECT * FROM Playlists;
                     INSERT INTO CorePlaylistEntries
+                        (EntryID, PlaylistID, TrackID, ViewOrder)
                         SELECT * FROM PlaylistEntries
                 ");
             } catch (Exception e) {
@@ -878,7 +1149,7 @@ namespace Banshee.Database
 
         private void OnServiceStarted (ServiceStartedArgs args)
         {
-            if (args.Service is UserJobManager) {
+            if (args.Service is JobScheduler) {
                 ServiceManager.ServiceStarted -= OnServiceStarted;
                 if (ServiceManager.SourceManager.MusicLibrary != null) {
                     RefreshMetadataDelayed ();
@@ -930,6 +1201,8 @@ namespace Banshee.Database
             }
 
             UserJob job = new UserJob (Catalog.GetString ("Refreshing Metadata"));
+            job.SetResources (Resource.Cpu, Resource.Disk, Resource.Database);
+            job.PriorityHints = PriorityHints.SpeedSensitive;
             job.Status = Catalog.GetString ("Scanning...");
             job.IconNames = new string [] { "system-search", "gtk-find" };
             job.Register ();
@@ -974,7 +1247,7 @@ namespace Banshee.Database
             }
 
             if (ServiceManager.DbConnection.Query<int> ("SELECT count(*) FROM CoreConfiguration WHERE Key = 'MetadataVersion'") == 0) {
-                Execute (String.Format ("INSERT INTO CoreConfiguration VALUES (null, 'MetadataVersion', {0})", CURRENT_METADATA_VERSION));
+                Execute (String.Format ("INSERT INTO CoreConfiguration (EntryID, Key, Value) VALUES (null, 'MetadataVersion', {0})", CURRENT_METADATA_VERSION));
             } else {
                 Execute (String.Format ("UPDATE CoreConfiguration SET Value = {0} WHERE Key = 'MetadataVersion'", CURRENT_METADATA_VERSION));
             }
@@ -986,4 +1259,16 @@ namespace Banshee.Database
 #endregion
 
     }
+
+    [SqliteFunction (Name = "BANSHEE_MIGRATE_PARTIAL", FuncType = FunctionType.Scalar, Arguments = 2)]
+    internal class MigratePartialFunction : SqliteFunction
+    {
+        public override object Invoke (object[] args)
+        {
+            string library_path = (string)args[0];
+            string filename_fragment = (string)args[1];
+            string full_path = Banshee.Base.Paths.Combine (library_path, filename_fragment);
+            return Banshee.Base.SafeUri.FilenameToUri (full_path);
+        }
+    }
 }
