@@ -35,6 +35,21 @@ namespace Banshee.PlaybackController
         Linear,
         Song,
         Artist,
-        Album
+        Album,
+        Rating,
+        Score
+    }
+
+    public class ShuffleModeChangedEventArgs : EventArgs
+    {
+        private PlaybackShuffleMode shuffle_mode;
+        public PlaybackShuffleMode ShuffleMode {
+            get { return shuffle_mode; }
+        }
+
+        public ShuffleModeChangedEventArgs (PlaybackShuffleMode shuffle_mode)
+        {
+            this.shuffle_mode = shuffle_mode;
+        }
     }
 }
