@@ -1,8 +1,9 @@
 //
 // PlaylistParser.cs
 //
-// Author:
+// Authors:
 //   Aaron Bockover <abockover@novell.com>
+//   Bertrand Lorentz <bertrand.lorentz@gmail.com>
 //
 // Copyright (C) 2007 Novell, Inc.
 //
@@ -41,11 +42,14 @@ namespace Banshee.Playlists.Formats
         private static PlaylistFormatDescription [] playlist_formats = new PlaylistFormatDescription [] {
             M3uPlaylistFormat.FormatDescription,
             PlsPlaylistFormat.FormatDescription,
-            AsxPlaylistFormat.FormatDescription
+            AsxPlaylistFormat.FormatDescription,
+            AsfReferencePlaylistFormat.FormatDescription,
+            XspfPlaylistFormat.FormatDescription
         };
         
         private List<Dictionary<string, object>> elements;
         private Uri base_uri = new Uri (Environment.CurrentDirectory);
+        private string title = null;
         
         public PlaylistParser ()
         {
@@ -56,7 +60,11 @@ namespace Banshee.Playlists.Formats
             ThreadAssist.AssertNotInMainThread ();
             lock (this) {
                 elements = null;
+                HttpWebResponse response = null;
                 Stream stream = null;
+                Stream web_stream = null;
+                bool partial_read = false;
+                long saved_position = 0;
                 
                 if (uri.Scheme == "file") {
                     stream = Banshee.IO.File.OpenRead (uri);
@@ -82,29 +90,31 @@ namespace Banshee.Playlists.Formats
                         request.Credentials = new NetworkCredential (username, password);
                     }
             
-                    HttpWebResponse response = (HttpWebResponse)request.GetResponse ();
-                    Stream web_stream = response.GetResponseStream ();
+                    response = (HttpWebResponse)request.GetResponse ();
+                    web_stream = response.GetResponseStream ();
                     
                     try {
                         stream = new MemoryStream ();
 
                         byte [] buffer = new byte[4096];
                         int read;
-            
-                        // If more than 4KB of data exists on an HTTP playlist, 
-                        // it's probably not a playlist. This kind of sucks,
-                        // but it should work until someone can prove otherwise
-                        
+
+                        // If we haven't read the whole stream and if
+                        // it turns out to be a playlist, we'll read the rest
                         read = web_stream.Read (buffer, 0, buffer.Length);
-                        if (read >= buffer.Length - 1) {
-                            throw new InvalidPlaylistException ();
+                        stream.Write (buffer, 0, read);
+                        if ((read = web_stream.Read (buffer, 0, buffer.Length)) > 0) {
+                            partial_read = true;
+                            stream.Write (buffer, 0, read);
+                            saved_position = stream.Position;
                         }
                         
-                        stream.Write (buffer, 0, read);
                         stream.Position = 0;
                     } finally {
-                        web_stream.Close ();
-                        response.Close ();
+                        if (!partial_read) {
+                            web_stream.Close ();
+                            response.Close ();
+                        }
                     }
                 } else {
                     Hyena.Log.DebugFormat ("Not able to parse playlist at {0}", uri);
@@ -127,6 +137,15 @@ namespace Banshee.Playlists.Formats
                     return false;
                 }
 
+                if (partial_read) {
+                    try {
+                        stream.Position = saved_position;
+                        Banshee.IO.StreamAssist.Save (web_stream, stream, false);
+                    } finally {
+                        web_stream.Close ();
+                        response.Close ();
+                    }
+                }
                 stream.Position = 0;
                 IPlaylistFormat playlist = (IPlaylistFormat)Activator.CreateInstance (matching_format.Type);
                 playlist.BaseUri = BaseUri;
@@ -134,6 +153,7 @@ namespace Banshee.Playlists.Formats
                 stream.Dispose ();
                 
                 elements = playlist.Elements;
+                Title = playlist.Title ?? Path.GetFileNameWithoutExtension (uri.LocalPath);
                 return true;
             }
         }
@@ -146,5 +166,10 @@ namespace Banshee.Playlists.Formats
             get { return base_uri; }
             set { base_uri = value; }
         }
+        
+        public string Title {
+            get { return title; }
+            set { title = value; }
+        }
     }
 }
