@@ -46,16 +46,16 @@ namespace Banshee.Gui.TrackEditor
         
         protected override void AddFields ()
         {
-            AddField (this, new TextEntry (), 
+            AddField (this, new TextEntry ("CoreTracks", "Composer"),
                 Catalog.GetString ("Set all composers to this value"),
-                delegate { return Catalog.GetString ("Composer:"); },
+                delegate { return Catalog.GetString ("C_omposer:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.Composer; },
                 delegate (EditorTrackInfo track, Widget widget) {  track.Composer = ((TextEntry)widget).Text; }
             );
             
-            AddField (this, new TextEntry (), 
+            AddField (this, new TextEntry ("CoreTracks", "Conductor"),
                 Catalog.GetString ("Set all conductors to this value"),
-                delegate { return Catalog.GetString ("Conductor:"); },
+                delegate { return Catalog.GetString ("Con_ductor:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.Conductor; },
                 delegate (EditorTrackInfo track, Widget widget) { track.Conductor = ((TextEntry)widget).Text; }
             );
@@ -65,31 +65,50 @@ namespace Banshee.Gui.TrackEditor
             box.Show ();
             PackStart (box, false, false, 0);
             
-            AddField (box, new TextEntry (), 
+            AddField (box, new TextEntry ("CoreTracks", "Grouping"),
                 Catalog.GetString ("Set all groupings to this value"),
-                delegate { return Catalog.GetString ("Grouping:"); },
+                delegate { return Catalog.GetString ("_Grouping:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.Grouping; },
                 delegate (EditorTrackInfo track, Widget widget) { track.Grouping = ((TextEntry)widget).Text; }
             );
-            
-            AddField (box, new SpinButtonEntry (0, 500, 1), 
+
+            SpinButtonEntry bpm_entry = new SpinButtonEntry (0, 999, 1);
+            bpm_entry.Digits = 0;
+            bpm_entry.MaxLength = 3;
+            bpm_entry.Numeric = true;
+            AddField (box, bpm_entry, 
                 Catalog.GetString ("Set all beats per minute to this value"),
-                delegate { return Catalog.GetString ("Beats Per Minute:"); },
+                delegate { return Catalog.GetString ("Bea_ts Per Minute:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((SpinButtonEntry)widget).Value = track.Bpm; },
-                delegate (EditorTrackInfo track, Widget widget) { track.Bpm = (int)((SpinButtonEntry)widget).Value; },
-                FieldOptions.Shrink
+                delegate (EditorTrackInfo track, Widget widget) { track.Bpm = ((SpinButtonEntry)widget).ValueAsInt; },
+                FieldOptions.Shrink | FieldOptions.NoSync
             );
+
+            HBox copyright_box = new HBox ();
+            copyright_box.Spacing = 12;
+            copyright_box.Show ();
+            PackStart (copyright_box, true, true, 0);
             
-            AddField (this, new TextEntry (), 
+            AddField (copyright_box, new TextEntry ("CoreTracks", "Copyright"),
                 Catalog.GetString ("Set all copyrights to this value"),
-                delegate { return Catalog.GetString ("Copyright:"); },
+                delegate { return Catalog.GetString ("Copyrig_ht:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.Copyright; },
                 delegate (EditorTrackInfo track, Widget widget) { track.Copyright = ((TextEntry)widget).Text; }
             );
-            
-            AddField (this, new TextViewEntry (), 
+
+            AddField (copyright_box, new LicenseEntry (),
+                Catalog.GetString ("Set all licenses to this value"),
+                delegate { return Catalog.GetString ("_License URI:"); },
+                delegate (EditorTrackInfo track, Widget widget) { ((LicenseEntry)widget).Value = track.LicenseUri; },
+                delegate (EditorTrackInfo track, Widget widget) { track.LicenseUri = ((LicenseEntry)widget).Value; }
+            );
+
+            TextViewEntry comment_entry = new TextViewEntry ();
+            comment_entry.HscrollbarPolicy = PolicyType.Automatic;
+            comment_entry.TextView.WrapMode = WrapMode.WordChar;
+            AddField (this, comment_entry, 
                 Catalog.GetString ("Set all comments to this value"),
-                delegate { return Catalog.GetString ("Comment:"); },
+                delegate { return Catalog.GetString ("Co_mment:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((TextViewEntry)widget).Text = track.Comment; },
                 delegate (EditorTrackInfo track, Widget widget) { track.Comment = ((TextViewEntry)widget).Text; }
             );
