@@ -66,13 +66,13 @@ namespace Banshee.Gui.Widgets
         private ImageSurface missing_audio_image;
         protected ImageSurface MissingAudioImage {
             get { return missing_audio_image ?? (missing_audio_image 
-                = new PixbufImageSurface (IconThemeUtils.LoadIcon (MissingIconSizeRequest, "audio-x-generic"), true)); }
+                = PixbufImageSurface.Create (IconThemeUtils.LoadIcon (MissingIconSizeRequest, "audio-x-generic"), true)); }
         }
         
         private ImageSurface missing_video_image;
         protected ImageSurface MissingVideoImage {
             get { return missing_video_image ?? (missing_video_image 
-                = new PixbufImageSurface (IconThemeUtils.LoadIcon (MissingIconSizeRequest, "video-x-generic"), true)); }
+                = PixbufImageSurface.Create (IconThemeUtils.LoadIcon (MissingIconSizeRequest, "video-x-generic"), true)); }
         }
         
         private Cairo.Color background_color;
@@ -173,12 +173,12 @@ namespace Banshee.Gui.Widgets
             text_light_color = Hyena.Gui.Theming.GtkTheme.GetCairoTextMidColor (this);
             
             if (missing_audio_image != null) {
-                missing_audio_image.Destroy ();
+                ((IDisposable)missing_audio_image).Dispose ();
                 missing_audio_image = null;
             }
 
             if (missing_video_image != null) {
-                missing_video_image.Destroy ();
+                ((IDisposable)missing_video_image).Dispose ();
                 missing_video_image = null;
             }
             
@@ -311,8 +311,10 @@ namespace Banshee.Gui.Widgets
         
         private void OnPlayerEvent (PlayerEventArgs args)
         {
-            if (args.Event == PlayerEvent.StartOfStream || args.Event == PlayerEvent.TrackInfoUpdated) {
+            if (args.Event == PlayerEvent.StartOfStream) {
                 LoadCurrentTrack ();
+            } else if (args.Event == PlayerEvent.TrackInfoUpdated) {
+                LoadCurrentTrack (true);
             } else if (args.Event == PlayerEvent.StateChange && (incoming_track != null || incoming_image != null)) {
                 PlayerEventStateChangeArgs state = (PlayerEventStateChangeArgs)args;
                 if (state.Current == PlayerState.Idle) {
@@ -340,12 +342,17 @@ namespace Banshee.Gui.Widgets
             idle_timeout_id = 0;
             return false;
         }
-        
+
         private void LoadCurrentTrack ()
+        {
+            LoadCurrentTrack (false);
+        }
+
+        private void LoadCurrentTrack (bool force_reload)
         {
             TrackInfo track = ServiceManager.PlayerEngine.CurrentTrack;
 
-            if (track == current_track && !IsMissingImage (current_image)) {
+            if (track == current_track && !IsMissingImage (current_image) && !force_reload) {
                 return;
             } else if (track == null) {
                 incoming_track = null;
@@ -400,7 +407,7 @@ namespace Banshee.Gui.Widgets
             }
             
             if (current_image != null && current_image != incoming_image && !IsMissingImage (current_image)) {
-                current_image.Destroy ();
+                ((IDisposable)current_image).Dispose ();
             }
             
             current_image = incoming_image;
