@@ -43,15 +43,14 @@ namespace Banshee.Gui
     public class PlaybackShuffleActions : BansheeActionGroup, IEnumerable<RadioAction>
     {
         private RadioAction active_action;
+        private RadioAction saved_action;
         private PlaybackActions playback_actions;
 
         public RadioAction Active {
             get { return active_action; }
             set {
                 active_action = value;
-                ShuffleMode.Set (active_action == null ? String.Empty : ActionNameToConfigId (active_action.Name));
                 ServiceManager.PlaybackController.ShuffleMode = (PlaybackShuffleMode)active_action.Value;
-                OnChanged ();
             }
         }
 
@@ -96,15 +95,28 @@ namespace Banshee.Gui
                 new RadioActionEntry ("ShuffleAlbumAction", null,
                     Catalog.GetString ("Shuffle by A_lbum"), null,
                     Catalog.GetString ("Play all songs from an album, then randomly choose another album"),
-                    (int)PlaybackShuffleMode.Album)
+                    (int)PlaybackShuffleMode.Album),
+
+                new RadioActionEntry ("ShuffleRatingAction", null,
+                    Catalog.GetString ("Shuffle by _Rating"), null,
+                    Catalog.GetString ("Play songs randomly, prefer higher rated songs"),
+                    (int)PlaybackShuffleMode.Rating),
+
+                new RadioActionEntry ("ShuffleScoreAction", null,
+                    Catalog.GetString ("Shuffle by S_core"), null,
+                    Catalog.GetString ("Play songs randomly, prefer higher scored songs"),
+                    (int)PlaybackShuffleMode.Score)
             }, 0, OnActionChanged);
                 
-            this["ShuffleOffAction"].IconName = "media-skip-forward";
+            this["ShuffleOffAction"].StockId = Gtk.Stock.MediaNext;
             this["ShuffleSongAction"].IconName = "media-playlist-shuffle";
             this["ShuffleArtistAction"].IconName = "media-playlist-shuffle";
             this["ShuffleAlbumAction"].IconName = "media-playlist-shuffle";
-            this["ShuffleArtistAction"].Sensitive = false;
-            this["ShuffleAlbumAction"].Sensitive = false;
+            this["ShuffleRatingAction"].IconName = "media-playlist-shuffle";
+            this["ShuffleScoreAction"].IconName = "media-playlist-shuffle";
+
+            ServiceManager.PlaybackController.ShuffleModeChanged += OnShuffleModeChanged;
+            ServiceManager.PlaybackController.SourceChanged += OnPlaybackSourceChanged;
 
             Gtk.Action action = this[ConfigIdToActionName (ShuffleMode.Get ())];
             if (action is RadioAction) {
@@ -116,6 +128,40 @@ namespace Banshee.Gui
             Active.Activate ();
         }
 
+        private void OnShuffleModeChanged (object o, ShuffleModeChangedEventArgs args)
+        {
+            if (active_action.Value != (int)args.ShuffleMode) {
+                // This happens only when changing the mode using DBus.
+                // In this case we need to locate the action by its value.
+                foreach (RadioAction action in this) {
+                    if (action.Value == (int)args.ShuffleMode) {
+                        active_action = action;
+                        break;
+                    }
+                }
+            }
+
+            if (saved_action == null) {
+                ShuffleMode.Set (ActionNameToConfigId (active_action.Name));
+            }
+            OnChanged();
+        }
+
+        private void OnPlaybackSourceChanged (object o, EventArgs args)
+        {
+            var source = ServiceManager.PlaybackController.Source;
+
+            if (saved_action == null && !source.CanShuffle) {
+                saved_action = Active;
+                Active = this["ShuffleOffAction"] as RadioAction;
+                Sensitive = false;
+            } else if (saved_action != null && source.CanShuffle) {
+                Active = saved_action;
+                saved_action = null;
+                Sensitive = true;
+            }
+        }
+
         private void OnActionChanged (object o, ChangedArgs args)
         {
             Active = args.Current;
@@ -123,6 +169,7 @@ namespace Banshee.Gui
 
         private void OnChanged ()
         {
+            playback_actions["NextAction"].StockId = Active.StockId;
             playback_actions["NextAction"].IconName = Active.IconName;
             EventHandler handler = Changed;
             if (handler != null) {
@@ -164,6 +211,8 @@ namespace Banshee.Gui
             yield return (RadioAction)this["ShuffleSongAction"];
             yield return (RadioAction)this["ShuffleArtistAction"];
             yield return (RadioAction)this["ShuffleAlbumAction"];
+            yield return (RadioAction)this["ShuffleRatingAction"];
+            yield return (RadioAction)this["ShuffleScoreAction"];
         }
 
         IEnumerator IEnumerable.GetEnumerator ()
@@ -186,7 +235,7 @@ namespace Banshee.Gui
             "playback", "shuffle_mode",
             "off",
             "Shuffle playback",
-            "Shuffle mode (shuffle_off, shuffle_song, shuffle_artist, shuffle_album)"
+            "Shuffle mode (shuffle_off, shuffle_song, shuffle_artist, shuffle_album, shuffle_rating, shuffle_score)"
         );
     }
 }
