@@ -57,6 +57,7 @@ namespace Banshee.Sources.Gui
                 return;
             }
             
+            renderer.Text = source.Name;
             renderer.Sensitive = source.CanActivate;
         }
         
@@ -145,6 +146,18 @@ namespace Banshee.Sources.Gui
             bool hide_counts = source.Count <= 0;
             
             Pixbuf icon = SourceIconResolver.ResolveIcon (source);
+
+            if (state == StateType.Insensitive) {
+                // Code ported from gtk_cell_renderer_pixbuf_render()
+                var icon_source = new IconSource () {
+                    Pixbuf = icon,
+                    Size = IconSize.SmallToolbar,
+                    SizeWildcarded = false
+                };
+
+                icon = widget.Style.RenderIcon (icon_source, widget.Direction, state,
+                    (IconSize)(-1), widget, "SourceRowRenderer");
+            }
             
             FontDescription fd = widget.PangoContext.FontDescription.Copy ();
             fd.Weight = (ISource)ServiceManager.PlaybackController.NextSource == (ISource)source 
@@ -162,12 +175,16 @@ namespace Banshee.Sources.Gui
             if (!hide_counts) {
                 count_layout = new Pango.Layout (widget.PangoContext);
                 count_layout.FontDescription = fd;
-                count_layout.SetMarkup (String.Format ("<span size=\"small\">({0})</span>", source.Count));
+                count_layout.SetMarkup (String.Format ("<span size=\"small\">{0}</span>", source.Count));
                 count_layout.GetPixelSize (out count_layout_width, out count_layout_height);
             }
 
             max_title_layout_width = cell_area.Width - (icon == null ? 0 : icon.Width) - count_layout_width - 10;
             
+            if (!hide_counts && max_title_layout_width < 0) {
+                hide_counts = true;
+            }
+			
             title_layout.FontDescription = fd;
             title_layout.Width = (int)(max_title_layout_width * Pango.Scale.PangoScale);
             title_layout.Ellipsize = EllipsizeMode.End;
