@@ -63,25 +63,28 @@ namespace Banshee.Lastfm.Radio
             AddImportant (
                 new ActionEntry (
                     "LastfmAddAction", Stock.Add,
-                     Catalog.GetString ("_Add Station"),
+                     Catalog.GetString ("_Add Station..."),
                      null, Catalog.GetString ("Add a new Last.fm radio station"), OnAddStation
                 )
             );
 
             Add (new ActionEntry [] {
+                new ActionEntry (
+                    "RefreshSourceAction", Stock.Refresh,
+                     Catalog.GetString ("Refresh"), null,
+                     String.Empty, OnRefreshSource
+                ),
+
                 new ActionEntry (
                     "LastfmConnectAction", null,
                      Catalog.GetString ("Connect"),
                      null, String.Empty, OnConnect
-                ),
-                new ActionEntry (
-                    "LastfmSortAction", "gtk-sort-descending",
-                    Catalog.GetString ("Sort Stations by"),
-                    null, String.Empty, null
                 )
             });
 
+            // Translators: {0} is a type of Last.fm station, eg "Fans of" or "Similar to".
             string listen_to = Catalog.GetString ("Listen to {0} Station");
+            // Translators: {0} is a type of Last.fm station, eg "Fans of" or "Similar to".
             string listen_to_long = Catalog.GetString ("Listen to the Last.fm {0} station for this artist");
 
             // Artist actions
@@ -157,31 +160,6 @@ namespace Banshee.Lastfm.Radio
             this["LastfmLoveAction"].IconName = "face-smile";
             this["LastfmHateAction"].IconName = "face-sad";
 
-            Add (
-                new RadioActionEntry [] {
-                    new RadioActionEntry (
-                        "LastfmSortStationsByNameAction", null,
-                         Catalog.GetString ("Station Name"),
-                         null, "", 0
-                    ),
-                    new RadioActionEntry (
-                        "LastfmSortStationsByPlayCountAction", null,
-                         Catalog.GetString ("Total Play Count"),
-                         null, "", 1
-                    ),
-                    new RadioActionEntry (
-                        "LastfmSortStationsByTypeAction", null,
-                         Catalog.GetString ("Station Type"),
-                         null, "", 2
-                    )
-                },
-                Array.IndexOf (LastfmSource.ChildComparers, lastfm.ChildComparer),
-                delegate (object sender, ChangedArgs args) {
-                    lastfm.ChildComparer = LastfmSource.ChildComparers[args.Current.Value];
-                    lastfm.SortChildSources ();
-                }
-            );
-
             this["LastfmLoveAction"].IsImportant = true;
             this["LastfmHateAction"].IsImportant = true;
 
@@ -201,7 +179,6 @@ namespace Banshee.Lastfm.Radio
         {
             Actions.UIManager.RemoveUi (actions_id);
             Actions.RemoveActionGroup (this);
-            RestoreShuffleRepeat ();
             ServiceManager.PlayerEngine.DisconnectEvent (OnPlayerEvent);
             base.Dispose ();
         }
@@ -231,6 +208,14 @@ namespace Banshee.Lastfm.Radio
             }
         }
 
+        private void OnRefreshSource (object o, EventArgs args)
+        {
+            StationSource source = Actions.SourceActions.ActionSource as StationSource;
+            if (source != null) {
+                source.Refresh();
+            }
+        }
+
         private void OnLoved (object sender, EventArgs args)
         {
             LastfmTrackInfo track = ServiceManager.PlayerEngine.CurrentTrack as LastfmTrackInfo;
@@ -360,7 +345,7 @@ namespace Banshee.Lastfm.Radio
         private void OnTrackRecommend (object sender, EventArgs args)
         {
         }
-
+        
 #endregion
 
         private string artist;
@@ -412,9 +397,8 @@ namespace Banshee.Lastfm.Radio
             }
 
             bool have_user = (lastfm.Account != null && lastfm.Account.UserName != null);
-            this["LastfmAddAction"].IsImportant = ServiceManager.PlaybackController.Source is LastfmSource;
             this["LastfmAddAction"].Sensitive = have_user;
-            this["LastfmSortAction"].Sensitive = have_user;
+            this["LastfmAddAction"].ShortLabel = Catalog.GetString ("_Add Station");
             this["LastfmConnectAction"].Visible = lastfm.Connection.State == ConnectionState.Disconnected;
 
             TrackInfo current_track = ServiceManager.PlayerEngine.CurrentTrack;
@@ -425,8 +409,6 @@ namespace Banshee.Lastfm.Radio
         }
 
         private uint track_actions_id;
-        private RadioAction old_shuffle;
-        private RadioAction old_repeat;
         private bool was_lastfm = false;
         private void OnPlaybackSourceChanged (object o, EventArgs args)
         {
@@ -437,26 +419,7 @@ namespace Banshee.Lastfm.Radio
 
             bool is_lastfm = ServiceManager.PlaybackController.Source is StationSource;
             Actions.PlaybackActions["PreviousAction"].Sensitive = !is_lastfm;
-            PlaybackRepeatActions repeat_actions = Actions.PlaybackActions.RepeatActions;
-            PlaybackShuffleActions shuffle_actions = Actions.PlaybackActions.ShuffleActions;
 
-            // Save/clear shuffle/repeat values when we first switch to a Last.fm station
-            if (is_lastfm && !was_lastfm) {
-                old_repeat = repeat_actions.Active;
-                repeat_actions.Active = repeat_actions["RepeatNoneAction"] as RadioAction;
-                
-                old_shuffle = shuffle_actions.Active;
-                shuffle_actions.Active = shuffle_actions["ShuffleOffAction"] as RadioAction;
-            }
-            // Restore shuffle/repeat values when we switch from a Last.fm station to a non Last.fm source
-            if (!is_lastfm && was_lastfm) {
-                RestoreShuffleRepeat ();
-            }
-            
-            // Set sensitivity
-            shuffle_actions.Sensitive = !is_lastfm;
-            repeat_actions.Sensitive = !is_lastfm;
-            
             if (is_lastfm && !was_lastfm)
                 track_actions_id = Actions.UIManager.AddUiFromResource ("LastfmTrackActions.xml");
             else if (!is_lastfm && was_lastfm)
@@ -464,14 +427,5 @@ namespace Banshee.Lastfm.Radio
 
             was_lastfm = is_lastfm;
         }
-
-        private void RestoreShuffleRepeat ()
-        {
-            if (Actions != null && Actions.PlaybackActions != null && old_repeat != null) {
-                Actions.PlaybackActions.RepeatActions.Active = old_repeat;
-                Actions.PlaybackActions.ShuffleActions.Active = old_shuffle;
-            }
-            old_repeat = old_shuffle = null;
-        }
     }
 }
