@@ -64,6 +64,7 @@ namespace Banshee.Podcasting.Data
                 PodcastTrackInfo pi = track.ExternalObject as PodcastTrackInfo;
                 if (pi != null) {
                     track.ReleaseDate = pi.PublishedDate;
+                    track.Enabled = !pi.Item.IsRead;
                 }
                 return pi;
             }
@@ -179,9 +180,9 @@ namespace Banshee.Podcasting.Data
             TrackInfo.PlaybackFinished += OnPlaybackFinished;
         }
 
-        private static void OnPlaybackFinished (TrackInfo track, double percentComplete)
+        private static void OnPlaybackFinished (TrackInfo track, double percentCompleted)
         {
-            if (percentComplete > 0.5 && track.PlayCount > 0) {
+            if (percentCompleted > 0.5 && track.PlayCount > 0) {
                 PodcastTrackInfo pi = PodcastTrackInfo.From (track);
                 if (pi != null && !pi.Item.IsRead) {
                     pi.Item.IsRead = true;
